package com.ishunwan.interactive_game_sdk;

import android.content.Context;

import com.ishunwan.player.interactivegame.SWGameCallback;
import com.ishunwan.player.interactivegame.SWGameClient;
import com.ishunwan.player.interactivegame.SWGameLib;
import com.ishunwan.player.interactivegame.data.SWCtrlPadInfo;
import com.ishunwan.player.interactivegame.data.SWPlayCloudGameInfo;
import com.ishunwan.player.interactivegame.data.SWUserInfo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * HelloPlugin
 */
public class InteractiveGameSdkPlugin implements FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler, SWGameCallback.IPlayChannelListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private final static String GAME_CLIENT_METHOD_CHANNEL_NAME = "com.ishunwan.interactive/game_client";
    private final static String GAME_CLIENT_EVENT_CHANNEL_NAME = "com.ishunwan.interactive/game_client_event";
    private final static String FLUTTER_PLAY_VIEW_TYPE = "com.ishunwan.interactive/game_player_view";

    private final static String GAME_CLIENT_INIT_FAIL_CODE = "101";
    private final static String JOIN_CHANNEL_FAIL_CODE = "102";
    private final static String LEAVE_CHANNEL_FAIL_CODE = "103";
    private final static String SET_PLAY_GAME_FAIL_CODE = "104";

    private final static String GAME_CLIENT_INIT_FAIL_MESSAGE = "gameClient初始化失败";
    private final static String JOIN_CHANNEL_FAIL_MESSAGE = "加入频道失败";
    private final static String LEAVE_CHANNEL_FAIL_MESSAGE = "离开频道失败";
    private final static String SET_PLAY_GAME_FAIL_MESSAGE = "设置游戏失败";

    private static MethodChannel sGameClientChannel;
    private static EventChannel sGameClientEventChannel;
    private static Context sContext;
    private static EventChannel.EventSink sSink;
    private static PlayViewFactory sPlayViewFactory;

    private SWGameClient mSWGameClient;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        sGameClientChannel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), GAME_CLIENT_METHOD_CHANNEL_NAME);
        sGameClientChannel.setMethodCallHandler(this);

        sGameClientEventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), GAME_CLIENT_EVENT_CHANNEL_NAME);
        sGameClientEventChannel.setStreamHandler(this);
        sContext = flutterPluginBinding.getApplicationContext();
        sPlayViewFactory = new PlayViewFactory(flutterPluginBinding.getFlutterEngine().getDartExecutor());
        flutterPluginBinding.getPlatformViewRegistry().registerViewFactory(FLUTTER_PLAY_VIEW_TYPE, sPlayViewFactory);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        sGameClientChannel = new MethodChannel(registrar.messenger(), GAME_CLIENT_METHOD_CHANNEL_NAME);
        sGameClientChannel.setMethodCallHandler(new InteractiveGameSdkPlugin());
        sGameClientEventChannel = new EventChannel(registrar.messenger(), GAME_CLIENT_EVENT_CHANNEL_NAME);
        sGameClientEventChannel.setStreamHandler(new InteractiveGameSdkPlugin());
        sContext = registrar.context();
        sPlayViewFactory = new PlayViewFactory(registrar.messenger());
        registrar.platformViewRegistry().registerViewFactory(FLUTTER_PLAY_VIEW_TYPE, sPlayViewFactory);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "init":
                init(call, result);
                break;
            case "debugMode":
                debugMode(call, result);
                break;
            case "joinChannel":
                joinChannel(call, result);
                break;
            case "setPlayCloudGame":
                setPlayGame(call,result);
                break;
            case "leaveChannel":
                leaveChannel(result);
                break;
            case "destroy":
                destroy(result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        sGameClientChannel.setMethodCallHandler(null);
        sGameClientEventChannel.setStreamHandler(null);
    }

    @Override
    public void onListen(Object o, EventChannel.EventSink eventSink) {
        sSink = eventSink;
    }

    @Override
    public void onCancel(Object o) {
        sSink = null;
    }

    public static void sendEvent(final String eventName, final HashMap<String, Object> map) {
        HashMap<String,Object> paramsMap = new HashMap<>();
        paramsMap.put("args",map);
        paramsMap.put("methodName", eventName);
        if (sSink != null) {
            sSink.success(paramsMap);
        }
    }

    private void init(MethodCall call, Result result) {
        boolean initResult = true;
        try {
            if (!SWGameLib.isInit()) {
                initResult = SWGameLib.init(sContext, String.valueOf(call.argument("appId")));
            }
            if (initResult) {
                if (mSWGameClient == null) {
                    mSWGameClient = new SWGameClient();
                    mSWGameClient.setPlayChannelListener(this);
                    sPlayViewFactory.setGameClient(mSWGameClient);
                }
                result.success(null);
            } else {
                resultError(result, GAME_CLIENT_INIT_FAIL_CODE, GAME_CLIENT_INIT_FAIL_MESSAGE);
            }
        } catch (Exception e) {
            resultError(result, GAME_CLIENT_INIT_FAIL_CODE, e.getMessage());
        }
    }

    private void debugMode(MethodCall call, Result result) {
        if (mSWGameClient != null) {
            try {
                SWGameLib.setDebug((boolean) call.argument("debugMode"));
                result.success(null);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            result.success(null);
        }
    }

    private void joinChannel(MethodCall call, Result result) {
        if (mSWGameClient != null) {
            try {
                String channelId = String.valueOf(call.argument("channelId"));
                String userId = String.valueOf(call.argument("uid"));
                if (call.argument("token") == null) {
                    mSWGameClient.joinChannel(channelId, userId);
                } else {
                    String token = String.valueOf(call.argument("token"));
                    mSWGameClient.joinChannel(channelId, userId, token);
                }
                result.success(null);
            } catch (Exception e) {
                resultError(result, JOIN_CHANNEL_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, JOIN_CHANNEL_FAIL_CODE, JOIN_CHANNEL_FAIL_MESSAGE);
        }
    }

    private void setPlayGame(MethodCall call,Result result) {
        if (mSWGameClient != null) {
            try {
                mSWGameClient.setPlayGame((int) call.argument("gameId"), (int) call.argument("padCount"));
                result.success(null);
            } catch (Exception e) {
                resultError(result, SET_PLAY_GAME_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, SET_PLAY_GAME_FAIL_CODE, SET_PLAY_GAME_FAIL_MESSAGE);
        }
    }

    private void leaveChannel(Result result) {
        if (mSWGameClient != null) {
            try {
                mSWGameClient.leaveChannel();
                result.success(null);
            } catch (Exception e) {
                resultError(result, LEAVE_CHANNEL_FAIL_CODE, e.getMessage());
            }
        } else {
            resultError(result, LEAVE_CHANNEL_FAIL_CODE, LEAVE_CHANNEL_FAIL_MESSAGE);
        }
    }

    private void destroy(Result result) {
        if (mSWGameClient != null) {
            try {
                mSWGameClient.destroy();
            } catch (Exception e) {
                e.printStackTrace();
            }
            mSWGameClient = null;
        }
        result.success(null);
    }

    private void resultError(MethodChannel.Result result, String code, String message) {
        Map<String, String> resultMap = new HashMap<>();
        resultMap.put("code", code);
        resultMap.put("message", message);
        result.success(resultMap);
    }

    @Override
    public void onJoinSuccess(String channelId) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("channel", channelId);
        sendEvent("onJoinChannelSuccess", map);
    }

    @Override
    public void onJoinFail(String channelId, int errorCode, String errorMessage) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("channel", channelId);
        map.put("errorCode", errorCode);
        map.put("errorMsg", errorMessage);
        sendEvent("onJoinChannelFailed", map);
    }

    @Override
    public void onDisconnect() {

    }

    @Override
    public void onUserListUpdate(List<SWUserInfo> list) {
        HashMap<String, Object> map = new HashMap<>();
        List<String> userList = new ArrayList<>(list.size());
        for (SWUserInfo swUserInfo : list) {
            userList.add(swUserInfo.getId());
        }
        map.put("userList", userList);
        sendEvent("onUserListUpdate", map);
    }

    @Override
    public void onCtrlPadUpdate(List<SWCtrlPadInfo> list) {
        HashMap<String, Object> map = new HashMap<>();
        List<Map<String, Object>> ctrlPadList = new ArrayList<>(list.size());
        for (SWCtrlPadInfo swCtrlPadInfo : list) {
            Map<String, Object> mapInfo = new HashMap<>();
            mapInfo.put("positionList", swCtrlPadInfo.getPositionList());
            mapInfo.put("userId", swCtrlPadInfo.getUserId());
            ctrlPadList.add(mapInfo);
        }
        map.put("ctrlPadList", ctrlPadList);
        sendEvent("onCtrlPadUpdate", map);
    }

    @Override
    public void onUserJoin(SWUserInfo swUserInfo) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("userId", swUserInfo.getId());
        sendEvent("onUserJoin", map);
    }

    @Override
    public void onUserLeave(SWUserInfo swUserInfo) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("userId", swUserInfo.getId());
        sendEvent("onUserLeave", map);
    }

    @Override
    public void onSetPlayGameSuccess(int gameId, int padCount) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("gameId", gameId);
        map.put("padCount", padCount);
        sendEvent("onSetPlayCloudGameSuccess", map);
    }

    @Override
    public void onSetPlayGameFail(int gameId, int padCount, int errorCode, String errorMsg) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("gameId", gameId);
        map.put("padCount", padCount);
        map.put("errorCode", errorCode);
        map.put("errorMsg", errorMsg);
        sendEvent("onSetPlayCloudGameFailed", map);
    }

    @Override
    public void onPlayGameUpdate(SWPlayCloudGameInfo swPlayCloudGameInfo) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("positionList", swPlayCloudGameInfo.getPositions());
        sendEvent("onPlayUpdate", map);
    }
}
