//
//  FlutterPlayerPlugin.m
//  interactive_game_sdk
//
//  Created by edz on 2020/9/16.
//

#import "FlutterPlayerPlugin.h"
#import "SWGamePlayer.h"
#import "SWPlayerView.h"
#import "FlutterMacro.h"

@interface FlutterPlayerPlugin ()<FlutterStreamHandler,SWGamePlayerDelegate>

@property (nonatomic, strong) FlutterMethodChannel* channel;

@property (nonatomic, strong) FlutterEventChannel* eventChannel;

@property (nonatomic, copy) FlutterEventSink eventSink;

@property (nonatomic, strong) FlutterPlayerView *playerView;

@property (nonatomic, assign) int64_t viewId;

@property (nonatomic, strong) SWGameClient *gameClient;

@property (nonatomic, strong) SWGamePlayer *gamePlayer;

@end

@implementation FlutterPlayerPlugin

- (void)dealloc {
    NSLog(@"dealloc_____%@",[self class]);
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
}

+ (void)createPlayerWithView:(FlutterPlayerView *)playerView
              viewIdentifier:(int64_t)viewId
             binaryMessenger:(NSObject<FlutterBinaryMessenger>*)messenger
                  gameClient:(nonnull SWGameClient *)gameClient {
    SWGamePlayer *gamePlayer = [gameClient createWithPlayView:(SWPlayerView *)playerView.view createFailed:^(NSError * _Nonnull error) {
        NSLog(@"error:%@",error);
    }];
    
    FlutterPlayerPlugin *playerPlugin = [[FlutterPlayerPlugin alloc] init];
    playerPlugin.playerView = playerView;
    playerPlugin.viewId = viewId;
    playerPlugin.gameClient = gameClient;
    playerPlugin.gamePlayer = gamePlayer;
    playerPlugin.gamePlayer.delegate = playerPlugin;
    
    NSString *channelName = [NSString stringWithFormat:@"com.ishunwan.interactive/game_player_%lld", viewId];
    NSString *eventChannelName = [NSString stringWithFormat:@"com.ishunwan.interactive/game_player_event_%lld", viewId];
    playerPlugin.channel = [FlutterMethodChannel methodChannelWithName:channelName binaryMessenger:messenger];
    playerPlugin.eventChannel = [FlutterEventChannel eventChannelWithName:eventChannelName binaryMessenger:messenger];
    [playerPlugin.eventChannel setStreamHandler:playerPlugin];
    
    __weak typeof(playerPlugin) weakPlayerPlugin = playerPlugin;
    [playerPlugin.channel setMethodCallHandler:^(FlutterMethodCall* call, FlutterResult result) {
        [weakPlayerPlugin onMethodCall:call result:result];
    }];
}

- (void)onMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"getPlatformVersion" isEqualToString:call.method]) {
      result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
    } else if ([@"startPlay" isEqualToString:call.method]) {
        NSLog(@"调用了startPlay方法 参数：%@___%@___%@",call.arguments,self.gamePlayer,self.playerView);
        if (self.gamePlayer) {
            int position = [call.arguments[@"position"] intValue];
            [self.gamePlayer startPlay:position Failed:^(NSError * _Nonnull error) {
                NSLog(@"error : %@",error.localizedDescription);
            }];
            result(nil);
        } else {
            result(@{@"message" : @"开始游戏失败", @"code" : @"201"});
        }
    } else if ([@"reStart" isEqualToString:call.method]) {
        NSLog(@"调用了reStart方法 参数：%@",call.arguments);
        [self.gamePlayer reStart];
        result(nil);
    } else if ([@"stopPlay" isEqualToString:call.method]) {
        NSLog(@"调用了stopPlay方法 参数：%@",call.arguments);
        [self.gamePlayer stopPlay];
        result(nil);
    } else if ([@"refreshFrame" isEqualToString:call.method]) {
        NSLog(@"调用了refreshFrame方法 参数：%@",call.arguments);
        //待定
//        [self performSelector:@selector(refreshFrame:) withObject:call.arguments afterDelay:0.5];
//        double width = [call.arguments[@"width"] doubleValue];
//        double height = [call.arguments[@"height"] doubleValue];
//        SWPlayerView *playerView = (SWPlayerView *)self.playerView.view;
//        [playerView refreshFrame:CGRectMake(0.f, 0.f, width, height)];
        result(nil);
    } else if ([@"grantCtrlPadUserId" isEqualToString:call.method]) {
        NSLog(@"调用了grantCtrlPadUserId方法 参数：%@",call.arguments);
        if (self.gamePlayer) {
            NSString *userId = call.arguments[@"userId"];
            [self.gamePlayer grantCtrlPadUserId:userId];
            result(nil);
        } else {
            result(@{@"message" : @"授予权限失败", @"code" : @"202"});
        }
    } else if ([@"revokeCtrlPadUserId" isEqualToString:call.method]) {
        NSLog(@"调用了revokeCtrlPadUserId方法 参数：%@",call.arguments);
        if (self.gamePlayer) {
            NSString *userId = call.arguments[@"userId"];
            [self.gamePlayer grantCtrlPadUserId:userId];
            result(nil);
        } else {
            result(@{@"message" : @"收回权限失败", @"code" : @"203"});
        }
    } else if ([@"sendBackCommand" isEqualToString:call.method]) {
        NSLog(@"调用了sendBackCommand方法 参数：%@",call.arguments);
        [self.gamePlayer sendControlCommand:SWGamePlayerControlCommandBack];
        result(nil);
    } else if ([@"sendHomeCommand" isEqualToString:call.method]) {
        NSLog(@"调用了sendHomeCommand方法 参数：%@",call.arguments);
        if (self.gamePlayer) {
            [self.gamePlayer sendControlCommand:SWGamePlayerControlCommandHome];
            result(nil);
        } else {
            result(@{@"message" : @"返回失败", @"code" : @"204"});
        }
    } else if ([@"destroy" isEqualToString:call.method]) {
        NSLog(@"调用了destroy方法 参数：%@____%@___%@",call.arguments,self.gamePlayer,self.playerView);
        [self.gamePlayer stopPlay];
        [self.playerView destroy];
        self.playerView = nil;
        self.gamePlayer.delegate = nil;
        self.gamePlayer = nil;
        self.gameClient = nil;
        result(nil);
    } else {
      result(FlutterMethodNotImplemented);
    }
}

- (void)refreshFrame:(id)arguments {
    //暂定
//    double width = [arguments[@"width"] doubleValue];
//    double height = [arguments[@"height"] doubleValue];
//    SWPlayerView *playerView = (SWPlayerView *)self.playerView.view;
//    NSLog(@"PlayerView_____%@",playerView);
//    [playerView refreshFrame:CGRectMake(0.f, 0.f, playerView.frame.size.width, playerView.frame.size.height)];
//    [playerView refreshFrame:CGRectMake(0.f, 0.f, width, height)];
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments eventSink:(FlutterEventSink)eventSink {
    self.eventSink = eventSink;
    return nil;
}
 
- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    return nil;
}

- (void)sendEvent:(NSString *)methodName params:(NSDictionary *)params {
    if (self.eventSink) {
        NSMutableDictionary *dict = [NSMutableDictionary new];
        [dict setObject:methodName forKey:@"methodName"];
        if (params) {
            [dict setObject:params forKey:@"args"];
        }
        self.eventSink([dict copy]);
    }
}

#pragma mark -- Delegate
- (void)playerDidStart:(SWGamePlayer *)player {
    [self sendEvent:@"onPlayStarted" params:@{}];
}

- (void)player:(SWGamePlayer *)player networkDelayChanged:(int)delayInMS {
    [self sendEvent:@"onPlayNetWorkDelay" params:@{@"delayInMS" : [NSNumber numberWithInt:delayInMS]}];
}

- (void)player:(SWGamePlayer *)player failed:(NSError *)error status:(int)status errorFrom:(int)errorFrom {
    [self sendEvent:@"onPlayError" params:@{@"status" : [NSNumber numberWithInteger:status], @"errorFrom" : [NSNumber numberWithInteger:errorFrom], @"errorCode" : [NSNumber numberWithInteger:error.code], @"errorMsg" : error.localizedDescription}];
}

- (void)ScreenOrientationChanged:(SWGamePlayer *)player portraitChanged:(BOOL)portrait {
    [self sendEvent:@"onPlayOrientationChanged" params:@{@"portrait" : [NSNumber numberWithBool:portrait]}];
}

@end
