//
//  FlutterPlayerView.m
//  interactive_game_sdk
//
//  Created by edz on 2020/9/15.
//

#import "FlutterPlayerView.h"
#import "FlutterPlayerPlugin.h"
#import "SWPlayerView.h"

@implementation FlutterPlayerViewFactory {
    NSObject<FlutterBinaryMessenger>* _messenger;
    SWGameClient *_gameClient;
}

- (instancetype)initWithMessenger:(NSObject<FlutterBinaryMessenger>*)messenger {
    self = [super init];
    if (self) {
        _messenger = messenger;
    }
    return self;
}

- (void)setGameClient:(SWGameClient *)gameClient {
    _gameClient = gameClient;
}

- (void)destroy {
    _gameClient = nil;
}

- (NSObject<FlutterMessageCodec>*)createArgsCodec {
    return [FlutterStandardMessageCodec sharedInstance];
}
    
- (NSObject<FlutterPlatformView>*)createWithFrame:(CGRect)frame
                                   viewIdentifier:(int64_t)viewId
                                        arguments:(id _Nullable)args {
    FlutterPlayerView *playerView = [[FlutterPlayerView alloc] initWithFrame:frame viewIdentifier:viewId arguments:args binaryMessenger:_messenger gameClient:_gameClient];
    return playerView;
}

@end

@interface FlutterPlayerView ()

@property (nonatomic, assign) int64_t viewId;

@property (nonatomic, strong) SWPlayerView *playerView;

@property (nonatomic, assign) CGRect originFrame;

@end

@implementation FlutterPlayerView

- (void)dealloc {
    NSLog(@"dealloc_____%@",[self class]);
}

- (instancetype)initWithFrame:(CGRect)frame
               viewIdentifier:(int64_t)viewId
                    arguments:(id _Nullable)args
              binaryMessenger:(NSObject<FlutterBinaryMessenger>*)messenger
                   gameClient:(nonnull SWGameClient *)gameClient {
    self = [super init];
    if (self) {
        self.viewId = viewId;
        
        self.playerView = [[SWPlayerView alloc] initWithFrame:frame];
        self.originFrame = self.playerView.frame;
        
        [FlutterPlayerPlugin createPlayerWithView:self viewIdentifier:viewId binaryMessenger:messenger gameClient:gameClient];
    }
    return self;
}

- (UIView *)view {
    if (!CGRectEqualToRect(self.originFrame, self.playerView.frame)) {
        NSLog(@"获取PlayerView_____%@",self.playerView);
        self.originFrame = self.playerView.frame;
        [self.playerView refreshFrame:CGRectMake(0.f, 0.f, self.playerView.frame.size.width, self.playerView.frame.size.height)];
    }
    return self.playerView;
}

- (void)destroy {
    [self.playerView removeFromSuperview];
    self.playerView = nil;
}

@end
