//
//  SWGamePlayer.h
//  PlaySDK
//
//  Created by BlingBling on 2020/9/8.
//  Copyright © 2020 iShunwan. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
NS_ASSUME_NONNULL_BEGIN
@class SWGamePlayer;

/** 画质等级 */
typedef NS_ENUM(NSInteger, SWGamePlayerPlayQualityLevel) {
    SWGamePlayerPlayQualityDefault = 0,
    SWGamePlayerPlayQualityHigh = 1,  //高清
    SWGamePlayerPlayQualityNormal = 2,//普通
    SWGamePlayerPlayQualityFast = 3,  //高速
    SWGamePlayerPlayQualityLow = 4,   //极速
    SWGamePlayerPlayQualityAuto = 5   //自动
};

/** 分辨率等级 */
typedef NS_ENUM(NSInteger, SWGamePlayerPlayResolutionLevel) {
    SWGamePlayerPlayResolutionHigh = 1,   //高分辨率
    SWGamePlayerPlayResolutionMiddle = 2, //中分辨率
    SWGamePlayerPlayResolutionNormal = 3, //普通分辨率
    SWGamePlayerPlayResolutionLow = 4,    //低分辨率
};

/** 按键控制指令 */
typedef NS_ENUM(NSInteger, SWGamePlayerControlCommand) {
    SWGamePlayerControlCommandBack,    //返回上一页
    SWGamePlayerControlCommandHome,    //回到桌面
    SWGamePlayerControlCommandShowTasks//切换多任务
};

/** 悬浮按钮位置 */
typedef NS_ENUM(NSInteger, SWGamePlayerFloatButtonPosition) {
    SWGamePlayerFloatButtonPositionLT, //左上
    SWGamePlayerFloatButtonPositionRT, //右上
    SWGamePlayerFloatButtonPositionLB, //左下
    SWGamePlayerFloatButtonPositionRB  //右下
};

@protocol SWGamePlayerDelegate<NSObject>
- (void)playerDidStart:(SWGamePlayer *)player;
- (void)player:(SWGamePlayer *)player qualityChanged:(SWGamePlayerPlayQualityLevel)playQuality;
- (void)player:(SWGamePlayer *)player resolutionChanged:(SWGamePlayerPlayResolutionLevel)resolutionLevel;
- (void)player:(SWGamePlayer *)player bitrateChanged:(int)bitrate;
- (void)player:(SWGamePlayer *)player fpsChanged:(int)playFps;
- (void)player:(SWGamePlayer *)player networkDelayChanged:(int)delayInMS;
- (void)player:(SWGamePlayer *)player failed:(NSError *)error status:(int)status errorFrom:(int)errorFrom;
- (void)playerDidStop:(SWGamePlayer *)player;
- (void)ScreenOrientationChanged:(SWGamePlayer *)player portraitChanged:(BOOL)portrait;
@end


@interface SWGamePlayer : NSObject
@property (nonatomic, weak,   nullable) id<SWGamePlayerDelegate> delegate;
+ (SWGamePlayer *)createPlayerWithPlayView:(UIView *)playView player:(NSObject *)player;
- (void)setManager:(NSObject *)manager delivery:(NSObject *)deliver channelID:(NSString *)channelID;
- (void)setupPkgDic:(NSDictionary *)pkgDic;
- (void)setPositonList:(NSArray *)positionList;
//control UI
- (void)setupPlayQuality:(SWGamePlayerPlayQualityLevel)quality;
- (void)setupLoadingView:(nullable UIView *)loadingView landscape:(nullable UIView *)loadingViewLand;
- (void)setupFloatView:(UIView *)floatView landscape:(nullable UIView *)floatViewLand;
- (void)setupFloatButtonWithSize:(CGSize)buttonSize
                           image:(UIImage *)image
                        selImage:(UIImage *)selImage
                    initPosition:(SWGamePlayerFloatButtonPosition)position;
-(void)setUPFloatButtonIsHidden:(BOOL)IsHidden;


- (void)startPlay:(int)position Failed:(void(^ _Nullable)(NSError * _Nonnull error))FailedBlock;
- (void)reStart;
- (void)stopPlay;
- (void)grantCtrlPadUserId:(NSString *)userId;
- (void)revokeCtrlPadUserId:(NSString *)userId;
//control funs
- (BOOL)changePlayQuality:(SWGamePlayerPlayQualityLevel)playQuality;
- (BOOL)changePlayResolution:(SWGamePlayerPlayResolutionLevel)playResolution;
- (BOOL)changePlayBitrate:(int)bitrate;
- (void)sendControlCommand:(SWGamePlayerControlCommand)command;
- (void)hideFloatViewAnimate:(void (^_Nullable)(BOOL finished))completion;
- (void)showDashboard:(BOOL)show;
@end

NS_ASSUME_NONNULL_END
