//
//  SWPlayerView.h
//  PlaySDK
//
//  Created by liuzhiying on 2018/5/7.
//  Copyright © 2018年 iShunwan. All rights reserved.
//


#import "SWBaseView.h"
#import "SWPlayer.h"


/** SWPlayerClientType: 客户端类型 */
typedef NS_ENUM(NSInteger, SWPlayerClientType) {
    SWPlayerClientTypePlay = 1,//试玩业务传
    SWPlayerClientTypeRedFinger = 2//红手指传
};

/** SWPlayerPlayType: 播放模式 */
typedef NS_ENUM(NSInteger, SWPlayerPlayType) {
    SWPlayerPlayTypeVideo = 1,//视频
    SWPlayerPlayTypeAudio = 2,//音频
    SWPlayerPlayTypeVideoAndAudio = 3 //音视频
};

@protocol SWPlayerVCDelegate <NSObject>
@required
- (void)checkCallback:(int)event;
- (void)playCallback:(int)event;
- (void)runCallback:(int)event;
- (void)playDidStart;
- (void)playWillStop:(NSError *)error status:(int)status errorFrom:(int)errorFrom;
- (void)playDidStop;
- (void)updateNetworkDelay:(int)delayInMS;
- (void)updateFrameFPS:(int)fps frameSize:(int)frameSize;
@optional
- (void)playQualityChanged:(SWPlayerPlayQualityLevel)playQuality autoChange:(BOOL)autoChange;
- (void)playResolutionChanged:(SWPlayerPlayResolutionLevel)playResolutionLevel;
- (void)playBitrateChanged:(int)bitrate;
- (void)playFpsChanged:(int)playFps;
@end

@protocol SWPlayerScreenDelegate <NSObject>
@required
- (void)onScreenOrientationChanged:(BOOL)portrait;
@end


@interface SWPlayerView : SWBaseView
@property (nonatomic,   weak) id <SWPlayerVCDelegate> delegate;
@property (nonatomic,   weak) id <SWPlayerScreenDelegate> screenDelegate;
@property (nonatomic, strong) UIView *floatView;
@property (nonatomic, strong) UIView *floatViewLand;
@property (nonatomic,   weak) UIView *loadingView;
@property (nonatomic,   weak) UIView *loadingViewLand;
@property (nonatomic, assign) int playBitrateValue;
@property (nonatomic, assign, readonly) BOOL voiceEnable;
@property (nonatomic, assign, readonly) BOOL autoChangeQuality;
@property (nonatomic, assign, readonly) CGAffineTransform viewTransform;
@property (nonatomic, assign, readonly) SWPlayerPlayResolutionLevel playResolutionLevel;
@property (nonatomic, assign, readonly) SWPlayerPlayQualityLevel playQualityLevel;

//UI
- (void)setupFloatButtonWithSize:(CGSize)buttonSize
                           image:(UIImage *)image
                        selImage:(UIImage *)selImage
                    initPosition:(SWPlayerFloatButtonPosition)initPosition;
-(void)setUPFloatButtonIsHidden:(BOOL)IsHidden;//默认为NO
//Property
- (void)setPlayQuality:(SWPlayerPlayQualityLevel)playQuality
      isAutoChangeMode:(BOOL)isAutoChangeQuality
              playType:(SWPlayerPlayType)playType
            clientType:(SWPlayerClientType)clientType;
- (BOOL)setPlayQuality:(SWPlayerPlayQualityLevel)quality autoChange:(BOOL)autoChange;
- (BOOL)setPlayResolution:(SWPlayerPlayResolutionLevel)playResolutionLevel;
- (BOOL)setPlayBitrate:(NSInteger)playBitrate;
//Control
- (void)cancelPlaying;
- (void)enableVoice:(BOOL)enable completion:(void(^)(BOOL enable))completion;
- (void)sendControlCommand:(SWPlayerControlCommand)command;
- (void)hideFloatViewAnimate:(void (^)(BOOL finished))completion;
- (void)showDashboard:(BOOL)show;
- (void)showMessage:(NSString *)message;
- (void)showHeadInfo:(NSString *)text animate:(BOOL)animate autoHide:(BOOL)autoHide;
//Play
- (void)playWithPackageName:(NSString *)packageName
                  appExtras:(NSString *)appExtras
                    padList:(NSString *)padList;
-(void)refreshFrame:(CGRect)frame;
@end
