import 'dart:async';

import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'events.dart';

class SWGamePlayer {
  MethodChannel _channel;

  EventChannel _eventChannel;

  GamePlayerChannelEventHandler _handler;

  Widget playerView;

  SWGamePlayer.createPlayerView() {
    playerView = this.createPlayerView();
  }

  Widget createPlayerView() {
    if (Platform.isIOS) {
      return UiKitView(
        viewType: 'com.ishunwan.interactive/game_player_view',
        onPlatformViewCreated: (viewId) {
          print('onPlatformView创建完成');
          _channel =
              MethodChannel('com.ishunwan.interactive/game_player_$viewId');
          _eventChannel = EventChannel(
              'com.ishunwan.interactive/game_player_event_$viewId');
          _eventChannel.receiveBroadcastStream().listen((event) {
            String methodName = event['methodName'];
            final argsMap = Map<String, dynamic>.from(event['args']);
            _handler?.process(this, methodName, argsMap);
          });
        },
      );
    } else {
      return AndroidView(
        viewType: 'com.ishunwan.interactive/game_player_view',
        onPlatformViewCreated: (viewId) {
          print('onPlatformView创建完成');
          _channel =
              MethodChannel('com.ishunwan.interactive/game_player_$viewId');
          _eventChannel = EventChannel(
              'com.ishunwan.interactive/game_player_event_$viewId');
          _eventChannel.receiveBroadcastStream().listen((event) {
            String methodName = event['methodName'];
            final argsMap = Map<String, dynamic>.from(event['args']);
            _handler?.process(this, methodName, argsMap);
          });
        },
      );
    }
  }

  void setEventHandler(GamePlayerChannelEventHandler handler) {
    _handler = handler;
  }

  Future<Map> startPlay(int position) async {
    final Map success =
        await _channel?.invokeMethod('startPlay', {'position': position});
    return success;
  }

  Future<Map> reStart() async {
    final Map success = await _channel?.invokeMethod('reStart');
    return success;
  }

  Future<Map> stopPlay() async {
    final Map success = await _channel?.invokeMethod('stopPlay');
    return success;
  }

  Future<Map> sendHomeCommand() async {
    final Map success = await _channel?.invokeMethod('sendHomeCommand');
    return success;
  }

  Future<Map> grantCtrlPadUserId(String userId) async {
    final Map success =
        await _channel?.invokeMethod('grantCtrlPadUserId', {'userId': userId});
    return success;
  }

  Future<Map> revokeCtrlPadUserId(String userId) async {
    final Map success =
        await _channel?.invokeMethod('revokeCtrlPadUserId', {'userId': userId});
    return success;
  }

  Future<Map> sendBackCommand() async {
    final Map success = await _channel?.invokeMethod('sendBackCommand');
    return success;
  }

  Future<Map> refreshFrame(double width, double height) async {
    final Map success = await _channel
        ?.invokeMethod('refreshFrame', {'width': width, 'height': height});
    return success;
  }

  Future<void> destroy() async {
    await _channel?.invokeMethod('destroy');
  }
}
