import 'dart:async';

import 'package:flutter/services.dart';

import 'events.dart';

class SWGameClient {
  static const MethodChannel _channel =
      const MethodChannel('com.ishunwan.interactive/game_client');
  static const EventChannel _eventChannel =
      const EventChannel('com.ishunwan.interactive/game_client_event');

  GameClientChannelEventHandler _handler;

  void setEventHandler(GameClientChannelEventHandler handler) {
    _handler = handler;
    _eventChannel.receiveBroadcastStream().listen((event) {
      String methodName = event['methodName'];
      final argsMap = Map<String, dynamic>.from(event['args']);
      _handler?.process(methodName, argsMap);
    });
  }

  Future<Map> init(String appId) async {
    final Map success = await _channel.invokeMethod('init', {'appId': appId});
    return success;
  }

  Future<void> debugMode(bool debugMode) async {
    await _channel.invokeMethod('debugMode', {'debugMode': debugMode});
  }

  Future<Map> joinChannel(String channelId, String uid, String token) async {
    final Map success = await _channel.invokeMethod(
        'joinChannel', {'channelId': channelId, 'uid': uid, 'token': token});
    return success;
  }

  Future<Map> setPlayCloudGame(int gameId) async {
    return this.setPlayCloudGameMulti(gameId, 1);
  }

  Future<Map> setPlayCloudGameMulti(int gameId, int padCount) async {
    final Map success = await _channel.invokeMethod(
        'setPlayCloudGame', {'gameId': gameId, 'padCount': padCount});
    return success;
  }

  Future<Map> leaveChannel() async {
    final Map success = await _channel.invokeMethod('leaveChannel');
    return success;
  }

  Future<void> destroy() async {
    await _channel.invokeMethod('destroy');
  }
}
